/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/**
 * User-token administrator user-profile view.
 *
 * @class Nexus.usertoken.AdminProfileView
 * @extends Ext.Panel
 * @namespace Nexus.usertoken
 */
NX.define('Nexus.usertoken.AdminProfileView', {
  extend: 'Ext.Panel',

  requires: [
    'Nexus.wonderland.AuthenticateButton'
  ],
  requirejs: ['Nexus/usertoken'],
  mixins: [
    'Nexus.LogAwareMixin'
  ],

  /**
   * @cfg {String} username
   */
  constructor: function (config) {
    var self = this;

    if (!config.username) {
      throw 'Missing config.username';
    }

    Ext.apply(self, {
      cls: 'nx-usertoken-AdminProfileView',
      border: false,
      items: [
        {
          xtype: 'fieldset',
          title: 'Reset',
          autoHeight: true,
          items: [
            {
              xtype: 'label',
              html: 'Resetting a users token will invalidate the current token and force a new token to be created the next time accessed by the user.<br/><br/>',
              cls: 'nx-usertoken-message'
            },
            {
              xtype: 'nx-wonderland-button-authenticate',
              text: 'Reset User Token',
              tooltip: 'Authenticate and reset this users token',
              listeners: {
                'authenticated': {
                  scope: self,
                  fn: self.resetUserToken
                }
              }
            }
          ]
        }
      ]
    });

    // Call super constructor
    self.constructor.superclass.constructor.apply(self, arguments);
  },

  getWaitMask: function () {
    var self = this;

    if (!self.waitMask) {
      self.waitMask = new Ext.LoadMask(self.getEl(), {
        msg: 'Please wait'
      });
    }
    return self.waitMask;
  },

  resetUserToken: function (button, authTicket) {
    var self = this;

    self.logDebug('Reset user token: ' + self.username);

    self.getWaitMask().show();

    Ext.Ajax.request({
      url: Nexus.usertoken.resourcePath('/users/' + self.username),
      method: 'DELETE',

      headers: {
        'X-NX-AuthTicket': authTicket
      },

      callback: function () {
        self.getWaitMask().hide();
      },

      success: function () {
        Nexus.usertoken.showMessage('User token has been reset for user: ' + self.username);
      }
    });
  },

  shouldShow: function () {
    var p = Sonatype.lib.Permissions;
    return Nexus.usertoken.enabled && p.checkPermission('nexus:usertoken:user', p.READ, p.DELETE);
  }

}, function () {
  var type = this;
  Nexus.profile.register(Nexus.usertoken.TITLE, type, ['admin']);
});

