/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext,Sonatype,Nexus*/
define('Sonatype/repoServer/ArtifactMetadataSearchContribution', function() {

// Add the custom metadata search
Sonatype.Events.addListener('searchTypeInit', function(searchTypes, panel) {
      var enterHandler = function(f, e) {
        if (e.getKey() == e.ENTER)
        {
          this.startSearch(this, true);
        }
      };

      var typeList = {
        equal : 'Equals',
        matches : 'Matches',
        bound : 'Key Defined',
        notEqual : 'Not Equal'
      };

      var metadataPopulator = function(panel, data) {
        // key
        if (data.length > 1)
        {
          panel.getTopToolbar().items.itemAt(2).setRawValue(data[1]);
        }
        // type
        if (data.length > 2)
        {
          panel.getTopToolbar().items.itemAt(4).value = data[2];
          panel.getTopToolbar().items.itemAt(4).setText(typeList[data[2]]);
        }
        // value
        if (data.length > 3)
        {
          panel.getTopToolbar().items.itemAt(6).setRawValue(data[3]);
        }
      };

      var m2FreeformStore = new Sonatype.SearchStore({
            grid : panel.grid,
            searchUrl : Sonatype.config.servicePath + '/search/m2/ngfreeform'
          });

      searchTypes.push({
            value : 'metadata',
            text : 'Metadata Search',
            scope : panel,
            store : m2FreeformStore,
            handler : panel.switchSearchType,
            quickSearchCheckHandler : function(panel, value) {
              return value.indexOf('=') > -1;
            },
            quickSearchHandler : function(panel, value) {
              var parts = value.split('=');
              var data = ['metadata'];

              if (parts.length == 2)
              {
                data.push(parts[0]);
                data.push('equal');
                data.push(parts[1]);
              }
              metadataPopulator(panel, data);
            },
            searchHandler : function(panel) {
              this.grid.store.baseParams = {};

              // key
              var v = panel.getTopToolbar().items.itemAt(2).getRawValue();
              if (v)
              {
                panel.grid.store.baseParams['p'] = v;
              }
              // type
              var v = panel.getTopToolbar().items.itemAt(4).value;
              if (v)
              {
                panel.grid.store.baseParams['t'] = v;
              }
              // value
              v = panel.getTopToolbar().items.itemAt(6).getRawValue();
              if (v)
              {
                panel.grid.store.baseParams['v'] = v;
              }

              if (panel.grid.store.baseParams['p'] == null)
              {
                panel.setWarningLabel('Key is required to run a search.');
                return;
              }

              panel.clearWarningLabel();

              panel.fetchRecords(panel);
            },
            applyBookmarkHandler : function(panel, data) {
              metadataPopulator(panel, data);
              panel.startSearch(this, false);
            },
            getBookmarkHandler : function(panel) {
              var result = panel.searchTypeButton.value;
              // key
              result += '~';
              var v = panel.getTopToolbar().items.itemAt(2).getRawValue();
              if (v)
              {
                result += v;
              }
              // type
              result += '~';
              v = panel.getTopToolbar().items.itemAt(4).value;
              if (v)
              {
                result += v;
              }
              // value
              result += '~';
              var v = panel.getTopToolbar().items.itemAt(6).getRawValue();
              if (v)
              {
                result += v;
              }
              return result;
            },
            panelItems : ['Key:', {
                  xtype : 'textfield',
                  id : 'metadatasearch-key',
                  size : 80,
                  listeners : {
                    'specialkey' : {
                      fn : enterHandler,
                      scope : panel
                    }
                  }
                }, {
                  xtype : 'tbspacer'
                }, {
                  id : 'metadatasearch-type',
                  text : 'Equals',
                  value : 'equal',
                  tooltip : 'Click for more search options',
                  scope : panel,
                  menu : {
                    items : [{
                          text : typeList.equal,
                          value : 'equal',
                          scope : panel,
                          handler : function() {
                            this.getTopToolbar().items.itemAt(4).value = 'equal';
                            this.getTopToolbar().items.itemAt(4).setText(typeList.equal);
                          }
                        }, {
                          text : typeList.matches,
                          value : 'matches',
                          scope : panel,
                          handler : function() {
                            this.getTopToolbar().items.itemAt(4).value = 'matches';
                            this.getTopToolbar().items.itemAt(4).setText(typeList.matches);
                          }
                        }, {
                          text : typeList.bound,
                          value : 'bound',
                          scope : panel,
                          handler : function() {
                            this.getTopToolbar().items.itemAt(4).value = 'bound';
                            this.getTopToolbar().items.itemAt(4).setText(typeList.bound);
                          }
                        }, {
                          text : typeList.notEqual,
                          value : 'notEqual',
                          scope : panel,
                          handler : function() {
                            this.getTopToolbar().items.itemAt(4).value = 'notEqual';
                            this.getTopToolbar().items.itemAt(4).setText(typeList.notEqual);
                          }
                        }]
                  }
                }, 'Value:', {
                  xtype : 'textfield',
                  id : 'metadatasearch-value',
                  size : 80,
                  listeners : {
                    'specialkey' : {
                      fn : enterHandler,
                      scope : panel
                    }
                  }
                }, {
                  xtype : 'tbspacer'
                }, {
                  icon : Sonatype.config.resourcePath + '/images/icons/search.gif',
                  cls : 'x-btn-icon',
                  scope : panel,
                  handler : function() {
                    this.startSearch(this, true);
                  }
                }]
          });
    });

});