/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX,Nexus,Ext,define,Sonatype*/
NX.define('Sonatype.repoServer.ArtifactProcurementPanel', {
  extend : 'Sonatype.panels.GridViewer',
  requires : ['Sonatype.repoServer.ArtifactProcurementStartPopup'],
  constructor : function(cfg) {
    var config = cfg || {};
    var defaultConfig = {
      title : 'Repositories'
    };
    Ext.apply(this, config, defaultConfig);

    this.groupStore = new Ext.data.JsonStore({
      root : 'data',
      id : 'id',
      fields : [
        {
          name : 'id'
        },
        {
          name : 'name'
        }
      ],
      url : Sonatype.config.repos.urls.groups,
      autoLoad : true,
      listeners : {
        load : {
          fn : this.groupStoreLoadHandler,
          scope : this
        },
        loadexception : {
          fn : this.groupStoreLoadHandler,
          scope : this
        }
      }
    });

    this.repositoryStore = new Ext.data.JsonStore({
      root : 'data',
      id : 'id',
      fields : [
        {
          name : 'id'
        },
        {
          name : 'name'
        },
        {
          name : 'repoPolicy'
        },
        {
          name : 'resourceURI'
        }
      ],
      url : Sonatype.config.repos.urls.allRepositories,
      autoLoad : false,
      listeners : {
        load : {
          fn : this.repositoryStoreLoadHandler,
          scope : this
        }
      }
    });

    Sonatype.repoServer.ArtifactProcurementPanel.superclass.constructor.call(this, {
      emptyText: 'No repositories defined',
      emptyTextWhileFiltering: 'No repositories matched criteria: {criteria}',

      rowClickEvent : 'procurementViewInit',
      rowContextClickEvent : 'repositoryMenuInit',
      url : Nexus.procurement.configuration.PROCUREMENT_SERVICE_PATH,
      dataAutoLoad : false,
      dataId : 'repositoryId',
      columns : [
        {
          name : 'repositoryId'
        },
        {
          name : 'resourceURI'
        },
        {
          name : 'targetClassId'
        },
        {
          name : 'targetId'
        },
        {
          name : 'id',
          mapping : 'repositoryId'
        },
        {
          name : 'name',
          sortType : Ext.data.SortTypes.asUCString,
          header : 'Repository',
          width : 175
        },
        {
          name : 'repoPolicy',
          header : 'Policy',
          width : 100
        },
        {
          name : 'procurementTargetName',
          header : 'Procured From',
          width : 150
        },
        {
          name : 'contentUri',
          header : 'Repository Path',
          autoExpand : true,
          renderer : function(s) {
            return '<a href="' + s + ((s != null && (s.charAt(s.length)) == '/') ? '' : '/') + '" target="_blank">'
                  + s + '</a>';
          }
        }
      ],
      tbar : [
        {
          text : 'Add Procured Repository',
          icon : Sonatype.config.resourcePath + '/images/icons/add.png',
          cls : 'x-btn-text-icon',
          scope : this,
          handler : this.addProcuredRepositoryHandler
        }
      ]
    });

    this.procurementStore = this.gridPanel.store;
    this.procurementStore.on('load', this.procurementStoreLoadHandler, this);
    this.procurementStore.on('loadexception', this.procurementStoreLoadHandler, this);
  },
  addProcuredRepositoryHandler : function(button, e) {
    new Sonatype.repoServer.ArtifactProcurementStartPopup();
  },

  groupStoreLoadHandler : function(store, records, options) {
    this.repositoryStore.reload();
  },

  procurementStoreLoadHandler : function(store, records, options) {
    if (records.length == undefined || records.length == 0) {
      this.dataStore.removeAll();
      return;
    }

    for (var i = records.length - 1; i >= 0; i--) {
      var rec = records[i];

      var groupTarget = rec.data.targetClassId == 'groups';
      var targetRec = (groupTarget ? this.groupStore : this.repositoryStore).getById(rec.data.targetId);
      var repositoryRec = this.repositoryStore.getById(rec.data.repositoryId);

      if (targetRec && repositoryRec) {
        rec.beginEdit();
        rec.set('name', repositoryRec.data.name);
        rec.set('repoPolicy', Sonatype.utils.upperFirstCharLowerRest(repositoryRec.data.repoPolicy));
        rec.set('procurementTargetName', targetRec.data.name);
        rec.set('resourceURI', repositoryRec.data.resourceURI);
        rec.set('contentUri', repositoryRec.data.resourceURI.replace(Sonatype.config.repos.urls.repositories,
              Sonatype.config.content.repositories));
        rec.commit();
        rec.endEdit();
      }
      else {
        store.remove(rec);
      }
    }
  },

  refreshHandler : function(button, e) {
    this.clearCards();
    this.groupStore.reload();
  },

  repositoryStoreLoadHandler : function(store, records, options) {
    this.procurementStore.reload();
  }
}, function() {
  Sonatype.Events.addListener('nexusNavigationInit', function(nexusPanel) {
    nexusPanel.add({
      enabled : Sonatype.lib.Permissions.checkPermission('nexus:procurementrepos', Sonatype.lib.Permissions.READ),
      sectionId : 'st-nexus-enterprise',
      title : 'Artifact Procurement',
      tabTitle : 'Procurement',
      tabId : 'artifact-procurement',
      tabCode : Sonatype.repoServer.ArtifactProcurementPanel
    });
  });

  Sonatype.Events.addListener('repositoryMenuInit', function(menu, rec) {
    if (rec.data.targetId != null && Sonatype.lib.Permissions.checkPermission('nexus:procurementrepos',
          Sonatype.lib.Permissions.DELETE)) {
      menu.add({
        text : 'Stop Procurement',
        handler : function(button, e) {
          Sonatype.MessageBox.show({
            animEl : Sonatype.view.mainTabPanel,
            title : 'Stop Procurement',
            msg : 'Stop procurement in ' + rec.data.name + '?',
            buttons : Sonatype.MessageBox.YESNO,
            scope : this,
            icon : Sonatype.MessageBox.QUESTION,
            fn : function(btnName) {
              if (btnName == 'yes' || btnName == 'ok') {
                Ext.Ajax.request({
                  method : 'DELETE',
                  url : Nexus.procurement.configuration.PROCUREMENT_SERVICE_PATH + '/' + rec.data.id,
                  callback : function(options, success, response) {
                    if (success) {
                      rec.store.remove(rec);
                      Sonatype.Events.fireEvent('repositoryChanged');

                      Sonatype.MessageBox.show({
                        title : 'Procurement Stopped',
                        msg : 'Procurement is now stopped in ' + rec.data.name + '.',
                        buttons : Sonatype.MessageBox.OK,
                        icon : Sonatype.MessageBox.INFO,
                        animEl : 'mb3'
                      });
                    }
                    else {
                      Sonatype.utils.connectionError(response,
                            'There was a problem stopping procurement in ' + rec.data.name + '.');
                    }
                  }
                });
              }
            }
          });
        }
      });
    }
  });

  Sonatype.repoServer.DefaultRepoHandler.repoActions.downloadRemoteItem.addCondition(
        function(repo) {
          return repo.targetId != null;
        }
  );
});


